package com.getswipe.textmatic.ui

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import com.getswipe.textmatic.R
import com.getswipe.textmatic.data.ForwardingRule
import com.getswipe.textmatic.databinding.ItemForwardingRuleBinding

class ForwardingRuleAdapter(
    private val onClickListener: (ForwardingRule) -> Unit
) : ListAdapter<ForwardingRule, ForwardingRuleViewHolder>(diffCallback) {

    fun getItemAt(position: Int) : ForwardingRule = getItem(position)

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ForwardingRuleViewHolder {
        val binding = ItemForwardingRuleBinding.inflate(
            LayoutInflater.from(parent.context), parent, false)
        return ForwardingRuleViewHolder(binding)
    }

    override fun onBindViewHolder(holder: ForwardingRuleViewHolder, position: Int) {
        val forwardingRule = getItem(position)
        holder.binding.iconImage.setImageResource(when (forwardingRule.direction) {
            ForwardingRuleViewModel.DIRECTION_INCOMING -> R.drawable.ic_baseline_arrow_right_alt_24
            ForwardingRuleViewModel.DIRECTION_OUTGOING -> R.drawable.ic_baseline_arrow_left_alt_24
            else -> R.drawable.ic_baseline_sync_alt_24
        })
        holder.binding.nameText.text = forwardingRule.name
        holder.binding.participantPatternText.text = forwardingRule.participantPattern ?: "n/a"
        holder.binding.contentPatternText.text = forwardingRule.contentPattern ?: "n/a"
        holder.binding.webhookUrlText.text = forwardingRule.webhookUrl
        holder.itemView.setOnClickListener { onClickListener(forwardingRule) }
    }
}

private val diffCallback = object : DiffUtil.ItemCallback<ForwardingRule>() {

    override fun areItemsTheSame(oldItem: ForwardingRule, newItem: ForwardingRule) =
        oldItem.id == newItem.id

    override fun areContentsTheSame(oldItem: ForwardingRule, newItem: ForwardingRule) =
        oldItem.name == newItem.name
                && oldItem.direction == newItem.direction
                && oldItem.participantPattern == newItem.participantPattern
                && oldItem.contentPattern == newItem.contentPattern
                && oldItem.webhookUrl == newItem.webhookUrl
}
